#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "AkelDLL.h"
#include "ConvFunc.h"
#include "Resources\Resource.h"


//Include conversion functions
#define xatoiA
#define xatoiW
#include "ConvFunc.h"

//Defines
#define STRID_CODEPAGE  1
#define STRID_NEWLINE   2
#define STRID_OK        3
#define STRID_CANCEL    4

#define CP_UNICODE_UCS2_LE  1200
#define CP_UNICODE_UCS2_BE  1201
#define CP_UNICODE_UTF7     65000
#define CP_UNICODE_UTF8     65001

#define STR_UNICODE_UCS2_LEA   "1200  (UTF-16 LE)"
#define STR_UNICODE_UCS2_LEW  L"1200  (UTF-16 LE)"
#define STR_UNICODE_UCS2_BEA   "1201  (UTF-16 BE)"
#define STR_UNICODE_UCS2_BEW  L"1201  (UTF-16 BE)"
#define STR_UNICODE_UTF8A      "65001 (UTF-8)"
#define STR_UNICODE_UTF8W     L"65001 (UTF-8)"
#define STR_UNICODE_UTF7A      "65000 (UTF-7)"
#define STR_UNICODE_UTF7W     L"65000 (UTF-7)"

//Functions prototypes
BOOL CALLBACK CodepageDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CodepageDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
void FillCodepageListboxA(HWND hWnd, int nCodePage);
void FillCodepageListboxW(HWND hWnd, int nCodePage);
void GetCodePageNameA(int nCodePage, char *szCodePage, int nLen);
void GetCodePageNameW(int nCodePage, wchar_t *wszCodePage, int nLen);
void ReadOptionsA();
void ReadOptionsW();
void SaveOptionsA();
void SaveOptionsW();
char* GetLangStringA(LANGID wLangID, int nStringID);
wchar_t* GetLangStringW(LANGID wLangID, int nStringID);
void InitCommon(PLUGINDATA *pd);

//GetProcAddress
BOOL (WINAPI *GetCPInfoExAPtr)(UINT, DWORD, LPCPINFOEXA);
BOOL (WINAPI *GetCPInfoExWPtr)(UINT, DWORD, LPCPINFOEXW);

//Global variables
char szPluginName[MAX_PATH];
wchar_t wszPluginName[MAX_PATH];
HINSTANCE hInstanceDLL;
HWND hMainWnd;
HWND hMdiClient;
HICON hMainIcon;
BOOL bOldWindows;
BOOL bMDI;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
BOOL bCodePageEnable=TRUE;
int nCodePage=0;
BOOL bBOM=TRUE;
BOOL bNewLineEnable=FALSE;
int nNewLine=NEWLINE_WIN;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="SaveAllAs";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=FALSE;
    return;
  }

  if (pd->bOnStart) return;
  if (!bInitCommon) InitCommon(pd);

  if (bOldWindows)
    DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_CODEPAGE), hMainWnd, (DLGPROC)CodepageDlgProcA);
  else
    DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_CODEPAGE), hMainWnd, (DLGPROC)CodepageDlgProcW);
}

BOOL CALLBACK CodepageDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndCodePageCheck;
  static HWND hWndCodePageList;
  static HWND hWndBOM;
  static HWND hWndNewLineCheck;
  static HWND hWndNewLineWin;
  static HWND hWndNewLineUnix;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndCodePageCheck=GetDlgItem(hDlg, IDC_CODEPAGE_CHECK);
    hWndCodePageList=GetDlgItem(hDlg, IDC_CODEPAGE_LIST);
    hWndBOM=GetDlgItem(hDlg, IDC_BOM);
    hWndNewLineCheck=GetDlgItem(hDlg, IDC_NEWLINE_CHECK);
    hWndNewLineWin=GetDlgItem(hDlg, IDC_NEWLINE_WIN);
    hWndNewLineUnix=GetDlgItem(hDlg, IDC_NEWLINE_UNIX);

    SetDlgItemTextA(hDlg, IDC_CODEPAGE_CHECK, GetLangStringA(wLangSystem, STRID_CODEPAGE));
    SetDlgItemTextA(hDlg, IDC_NEWLINE_CHECK, GetLangStringA(wLangSystem, STRID_NEWLINE));
    SetDlgItemTextA(hDlg, IDOK, GetLangStringA(wLangSystem, STRID_OK));
    SetDlgItemTextA(hDlg, IDCANCEL, GetLangStringA(wLangSystem, STRID_CANCEL));

    FillCodepageListboxA(hWndCodePageList, nCodePage);

    if (nCodePage == CP_UNICODE_UCS2_LE ||
        nCodePage == CP_UNICODE_UCS2_BE ||
        nCodePage == CP_UNICODE_UTF8)
    {
      if (bBOM)
        SendMessage(hWndBOM, BM_SETCHECK, BST_CHECKED, 0);
    }
    if (bCodePageEnable)
      SendMessage(hWndCodePageCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (bNewLineEnable)
      SendMessage(hWndNewLineCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (nNewLine == NEWLINE_WIN)
      SendMessage(hWndNewLineWin, BM_SETCHECK, BST_CHECKED, 0);
    else if (nNewLine == NEWLINE_UNIX)
      SendMessage(hWndNewLineUnix, BM_SETCHECK, BST_CHECKED, 0);

    SendMessage(hDlg, WM_COMMAND, IDC_CODEPAGE_CHECK, 0);
    SendMessage(hDlg, WM_COMMAND, IDC_NEWLINE_CHECK, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_CODEPAGE_CHECK)
    {
      if (SendMessage(hWndCodePageCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
      {
        EnableWindow(hWndCodePageList, TRUE);

        if (nCodePage == CP_UNICODE_UCS2_LE ||
            nCodePage == CP_UNICODE_UCS2_BE ||
            nCodePage == CP_UNICODE_UTF8)
        {
          EnableWindow(hWndBOM, TRUE);
        }
        else
        {
          EnableWindow(hWndBOM, FALSE);
        }
      }
      else
      {
        EnableWindow(hWndCodePageList, FALSE);
        EnableWindow(hWndBOM, FALSE);
      }
    }
    else if (LOWORD(wParam) == IDC_CODEPAGE_LIST && HIWORD(wParam) == CBN_SELCHANGE)
    {
      nCodePage=GetDlgItemInt(hDlg, IDC_CODEPAGE_LIST, NULL, FALSE);

      if (nCodePage == CP_UNICODE_UCS2_LE ||
          nCodePage == CP_UNICODE_UCS2_BE ||
          nCodePage == CP_UNICODE_UTF8)
      {
        SendMessage(hWndBOM, BM_SETCHECK, BST_CHECKED, 0);
        EnableWindow(hWndBOM, TRUE);
      }
      else
      {
        SendMessage(hWndBOM, BM_SETCHECK, BST_UNCHECKED, 0);
        EnableWindow(hWndBOM, FALSE);
      }
    }
    else if (LOWORD(wParam) == IDC_NEWLINE_CHECK)
    {
      if (SendMessage(hWndNewLineCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
      {
        EnableWindow(hWndNewLineWin, TRUE);
        EnableWindow(hWndNewLineUnix, TRUE);
      }
      else
      {
        EnableWindow(hWndNewLineWin, FALSE);
        EnableWindow(hWndNewLineUnix, FALSE);
      }
    }
    else if (LOWORD(wParam) == IDOK)
    {
      EDITINFO ei;
      HWND hWndFrameFirst;
      HWND hWndFrameActive;

      bCodePageEnable=FALSE;
      nCodePage=0;
      bBOM=FALSE;
      bNewLineEnable=FALSE;
      nNewLine=NEWLINE_WIN;

      if (SendMessage(hWndCodePageCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        bCodePageEnable=TRUE;

      nCodePage=GetDlgItemInt(hDlg, IDC_CODEPAGE_LIST, NULL, FALSE);

      if (SendMessage(hWndBOM, BM_GETCHECK, 0, 0) == BST_CHECKED)
        bBOM=TRUE;

      if (SendMessage(hWndNewLineCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        bNewLineEnable=TRUE;

      if (SendMessage(hWndNewLineUnix, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nNewLine=NEWLINE_UNIX;

      SaveOptionsA();
      EndDialog(hDlg, 0);

      //Save documents
      if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
      {
        if (hWndFrameFirst=hWndFrameActive=GetParent(ei.hWndEdit))
        {
          do
          {
            SAVEDOCUMENTA sd;

            if ((bNewLineEnable && nNewLine != ei.nNewLine) ||
                (bCodePageEnable && ((nCodePage != ei.nCodePage || bBOM != ei.bBOM) ||
                                     (nCodePage == ei.nCodePage && ei.bModified))))
            {
              if (bNewLineEnable)
                SendMessage(hMainWnd, AKD_SETNEWLINE, (WPARAM)ei.hWndEdit, nNewLine);
              lstrcpynA(sd.szFile, (char *)ei.pFile, MAX_PATH);
              sd.nCodePage=bCodePageEnable?nCodePage:ei.nCodePage;
              sd.bBOM=bCodePageEnable?bBOM:ei.bBOM;
              sd.dwFlags=SD_UPDATE;
              if (SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd))
                break;
            }
            if (!bMDI) break;

            SendMessage(hMdiClient, WM_MDINEXT, (WPARAM)hWndFrameActive, FALSE);
            SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei);
            hWndFrameActive=GetParent(ei.hWndEdit);
          }
          while (hWndFrameActive != hWndFrameFirst);
        }
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK CodepageDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndCodePageCheck;
  static HWND hWndCodePageList;
  static HWND hWndBOM;
  static HWND hWndNewLineCheck;
  static HWND hWndNewLineWin;
  static HWND hWndNewLineUnix;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndCodePageCheck=GetDlgItem(hDlg, IDC_CODEPAGE_CHECK);
    hWndCodePageList=GetDlgItem(hDlg, IDC_CODEPAGE_LIST);
    hWndBOM=GetDlgItem(hDlg, IDC_BOM);
    hWndNewLineCheck=GetDlgItem(hDlg, IDC_NEWLINE_CHECK);
    hWndNewLineWin=GetDlgItem(hDlg, IDC_NEWLINE_WIN);
    hWndNewLineUnix=GetDlgItem(hDlg, IDC_NEWLINE_UNIX);

    SetDlgItemTextW(hDlg, IDC_CODEPAGE_CHECK, GetLangStringW(wLangSystem, STRID_CODEPAGE));
    SetDlgItemTextW(hDlg, IDC_NEWLINE_CHECK, GetLangStringW(wLangSystem, STRID_NEWLINE));
    SetDlgItemTextW(hDlg, IDOK, GetLangStringW(wLangSystem, STRID_OK));
    SetDlgItemTextW(hDlg, IDCANCEL, GetLangStringW(wLangSystem, STRID_CANCEL));

    FillCodepageListboxW(hWndCodePageList, nCodePage);

    if (nCodePage == CP_UNICODE_UCS2_LE ||
        nCodePage == CP_UNICODE_UCS2_BE ||
        nCodePage == CP_UNICODE_UTF8)
    {
      if (bBOM)
        SendMessage(hWndBOM, BM_SETCHECK, BST_CHECKED, 0);
    }
    if (bCodePageEnable)
      SendMessage(hWndCodePageCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (bNewLineEnable)
      SendMessage(hWndNewLineCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (nNewLine == NEWLINE_WIN)
      SendMessage(hWndNewLineWin, BM_SETCHECK, BST_CHECKED, 0);
    else if (nNewLine == NEWLINE_UNIX)
      SendMessage(hWndNewLineUnix, BM_SETCHECK, BST_CHECKED, 0);

    SendMessage(hDlg, WM_COMMAND, IDC_CODEPAGE_CHECK, 0);
    SendMessage(hDlg, WM_COMMAND, IDC_NEWLINE_CHECK, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_CODEPAGE_CHECK)
    {
      if (SendMessage(hWndCodePageCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
      {
        EnableWindow(hWndCodePageList, TRUE);

        if (nCodePage == CP_UNICODE_UCS2_LE ||
            nCodePage == CP_UNICODE_UCS2_BE ||
            nCodePage == CP_UNICODE_UTF8)
        {
          EnableWindow(hWndBOM, TRUE);
        }
        else
        {
          EnableWindow(hWndBOM, FALSE);
        }
      }
      else
      {
        EnableWindow(hWndCodePageList, FALSE);
        EnableWindow(hWndBOM, FALSE);
      }
    }
    else if (LOWORD(wParam) == IDC_CODEPAGE_LIST && HIWORD(wParam) == CBN_SELCHANGE)
    {
      nCodePage=GetDlgItemInt(hDlg, IDC_CODEPAGE_LIST, NULL, FALSE);

      if (nCodePage == CP_UNICODE_UCS2_LE ||
          nCodePage == CP_UNICODE_UCS2_BE ||
          nCodePage == CP_UNICODE_UTF8)
      {
        SendMessage(hWndBOM, BM_SETCHECK, BST_CHECKED, 0);
        EnableWindow(hWndBOM, TRUE);
      }
      else
      {
        SendMessage(hWndBOM, BM_SETCHECK, BST_UNCHECKED, 0);
        EnableWindow(hWndBOM, FALSE);
      }
    }
    else if (LOWORD(wParam) == IDC_NEWLINE_CHECK)
    {
      if (SendMessage(hWndNewLineCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
      {
        EnableWindow(hWndNewLineWin, TRUE);
        EnableWindow(hWndNewLineUnix, TRUE);
      }
      else
      {
        EnableWindow(hWndNewLineWin, FALSE);
        EnableWindow(hWndNewLineUnix, FALSE);
      }
    }
    else if (LOWORD(wParam) == IDOK)
    {
      EDITINFO ei;
      HWND hWndFrameFirst;
      HWND hWndFrameActive;

      bCodePageEnable=FALSE;
      nCodePage=0;
      bBOM=FALSE;
      bNewLineEnable=FALSE;
      nNewLine=NEWLINE_WIN;

      if (SendMessage(hWndCodePageCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        bCodePageEnable=TRUE;

      nCodePage=GetDlgItemInt(hDlg, IDC_CODEPAGE_LIST, NULL, FALSE);

      if (SendMessage(hWndBOM, BM_GETCHECK, 0, 0) == BST_CHECKED)
        bBOM=TRUE;

      if (SendMessage(hWndNewLineCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        bNewLineEnable=TRUE;

      if (SendMessage(hWndNewLineUnix, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nNewLine=NEWLINE_UNIX;

      SaveOptionsW();
      EndDialog(hDlg, 0);

      //Save documents
      if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
      {
        if (hWndFrameFirst=hWndFrameActive=GetParent(ei.hWndEdit))
        {
          do
          {
            SAVEDOCUMENTW sd;

            if ((bNewLineEnable && nNewLine != ei.nNewLine) ||
                (bCodePageEnable && ((nCodePage != ei.nCodePage || bBOM != ei.bBOM) ||
                                     (nCodePage == ei.nCodePage && ei.bModified))))
            {
              if (bNewLineEnable)
                SendMessage(hMainWnd, AKD_SETNEWLINE, (WPARAM)ei.hWndEdit, nNewLine);
              lstrcpynW(sd.szFile, (wchar_t *)ei.pFile, MAX_PATH);
              sd.nCodePage=bCodePageEnable?nCodePage:ei.nCodePage;
              sd.bBOM=bCodePageEnable?bBOM:ei.bBOM;
              sd.dwFlags=SD_UPDATE;
              if (SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd))
                break;
            }
            if (!bMDI) break;

            SendMessage(hMdiClient, WM_MDINEXT, (WPARAM)hWndFrameActive, FALSE);
            SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei);
            hWndFrameActive=GetParent(ei.hWndEdit);
          }
          while (hWndFrameActive != hWndFrameFirst);
        }
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

void FillCodepageListboxA(HWND hWnd, int nCodePage)
{
  char szCodepageName[MAX_PATH];
  int *lpCodepageList;
  int nSel=0;
  int nIndex=0;
  int i;

  if (lpCodepageList=(int *)SendMessage(hMainWnd, AKD_GETCODEPAGELIST, 0, 0))
  {
    for (i=0; lpCodepageList[i]; ++i)
    {
      if (nCodePage == lpCodepageList[i]) nSel=nIndex;

      GetCodePageNameA(lpCodepageList[i], szCodepageName, MAX_PATH);
      SendMessageA(hWnd, CB_ADDSTRING, 0, (LPARAM)szCodepageName);
      ++nIndex;
    }
    if (!nSel) nCodePage=lpCodepageList[0];
    SendMessage(hWnd, CB_SETCURSEL, nSel, 0);
  }
}

void FillCodepageListboxW(HWND hWnd, int nCodePage)
{
  wchar_t wszCodepageName[MAX_PATH];
  int *lpCodepageList;
  int nSel=0;
  int nIndex=0;
  int i;

  if (lpCodepageList=(int *)SendMessage(hMainWnd, AKD_GETCODEPAGELIST, 0, 0))
  {
    for (i=0; lpCodepageList[i]; ++i)
    {
      if (nCodePage == lpCodepageList[i]) nSel=nIndex;

      GetCodePageNameW(lpCodepageList[i], wszCodepageName, MAX_PATH);
      SendMessageW(hWnd, CB_ADDSTRING, 0, (LPARAM)wszCodepageName);
      ++nIndex;
    }
    if (!nSel) nCodePage=lpCodepageList[0];
    SendMessage(hWnd, CB_SETCURSEL, nSel, 0);
  }
}

void GetCodePageNameA(int nCodePage, char *szCodePage, int nLen)
{
  CPINFOEXA CPInfoEx;
  char szNumber[32];

  if (nCodePage)
  {
    if (nCodePage == CP_UNICODE_UCS2_LE)
      lstrcpynA(szCodePage, STR_UNICODE_UCS2_LEA, nLen);
    else if (nCodePage == CP_UNICODE_UCS2_BE)
      lstrcpynA(szCodePage, STR_UNICODE_UCS2_BEA, nLen);
    else if (nCodePage == CP_UNICODE_UTF8)
      lstrcpynA(szCodePage, STR_UNICODE_UTF8A, nLen);
    else if (nCodePage == CP_UNICODE_UTF7)
      lstrcpynA(szCodePage, STR_UNICODE_UTF7A, nLen);
    else
    {
      if (GetCPInfoExAPtr && (*GetCPInfoExAPtr)(nCodePage, 0, &CPInfoEx) && nCodePage == xatoiA(CPInfoEx.CodePageName))
      {
        lstrcpynA(szCodePage, CPInfoEx.CodePageName, nLen);
      }
      else
      {
        wsprintfA(szNumber, "%d ", nCodePage);
        lstrcpynA(szCodePage, szNumber, nLen);
      }
    }
  }
  else szCodePage[0]='\0';
}

void GetCodePageNameW(int nCodePage, wchar_t *wszCodePage, int nLen)
{
  CPINFOEXW CPInfoExW;
  wchar_t wszNumber[32];

  if (nCodePage)
  {
    if (nCodePage == CP_UNICODE_UCS2_LE)
      lstrcpynW(wszCodePage, STR_UNICODE_UCS2_LEW, nLen);
    else if (nCodePage == CP_UNICODE_UCS2_BE)
      lstrcpynW(wszCodePage, STR_UNICODE_UCS2_BEW, nLen);
    else if (nCodePage == CP_UNICODE_UTF8)
      lstrcpynW(wszCodePage, STR_UNICODE_UTF8W, nLen);
    else if (nCodePage == CP_UNICODE_UTF7)
      lstrcpynW(wszCodePage, STR_UNICODE_UTF7W, nLen);
    else
    {
      if (GetCPInfoExWPtr && (*GetCPInfoExWPtr)(nCodePage, 0, &CPInfoExW) && nCodePage == xatoiW(CPInfoExW.CodePageName))
      {
        lstrcpynW(wszCodePage, CPInfoExW.CodePageName, nLen);
      }
      else
      {
        wsprintfW(wszNumber, L"%d ", nCodePage);
        lstrcpynW(wszCodePage, wszNumber, nLen);
      }
    }
  }
  else wszCodePage[0]='\0';
}


//// Options

void ReadOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)szPluginName))
  {
    po.pOptionName="CodePageEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCodePageEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="CodePage";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nCodePage;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="BOM";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bBOM;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="NewLineEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bNewLineEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="NewLine";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nNewLine;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void ReadOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"CodePageEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCodePageEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"CodePage";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nCodePage;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"BOM";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bBOM;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"NewLineEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bNewLineEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"NewLine";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nNewLine;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)szPluginName))
  {
    po.pOptionName="CodePageEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCodePageEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="CodePage";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nCodePage;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="BOM";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bBOM;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="NewLineEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bNewLineEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="NewLine";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nNewLine;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"CodePageEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCodePageEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"CodePage";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nCodePage;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"BOM";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bBOM;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"NewLineEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bNewLineEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"NewLine";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nNewLine;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

char* GetLangStringA(LANGID wLangID, int nStringID)
{
  static char szStringBuf[MAX_PATH];

  WideCharToMultiByte(CP_ACP, 0, GetLangStringW(wLangID, nStringID), -1, szStringBuf, MAX_PATH, NULL, NULL);
  return szStringBuf;
}

wchar_t* GetLangStringW(LANGID wLangID, int nStringID)
{
  if (wLangID == LANG_RUSSIAN)
  {
    if (nStringID == STRID_CODEPAGE)
      return L"\x041A\x043E\x0434\x0438\x0440\x043E\x0432\x043A\x0430";
    if (nStringID == STRID_NEWLINE)
      return L"\x0424\x043E\x0440\x043C\x0430\x0442\x0020\x043D\x043E\x0432\x043E\x0439\x0020\x0441\x0442\x0440\x043E\x043A\x0438";
    if (nStringID == STRID_OK)
      return L"\x004F\x004B";
    if (nStringID == STRID_CANCEL)
      return L"\x041E\x0442\x043C\x0435\x043D\x0430";
  }
  else
  {
    if (nStringID == STRID_CODEPAGE)
      return L"Codepage";
    if (nStringID == STRID_NEWLINE)
      return L"New line format";
    if (nStringID == STRID_OK)
      return L"OK";
    if (nStringID == STRID_CANCEL)
      return L"Cancel";
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hInstanceDLL=pd->hInstanceDLL;
  hMainWnd=pd->hMainWnd;
  hMdiClient=pd->hMdiClient;
  hMainIcon=pd->hMainIcon;
  bOldWindows=pd->bOldWindows;
  bMDI=pd->bMDI;
  wLangSystem=PRIMARYLANGID(pd->wLangSystem);

  if (bOldWindows)
  {
    int i;

    for (i=0; ((char *)pd->pFunction)[i] != ':'; ++i)
      szPluginName[i]=((char *)pd->pFunction)[i];
    szPluginName[i]='\0';
  }
  else
  {
    int i;

    for (i=0; ((wchar_t *)pd->pFunction)[i] != ':'; ++i)
      wszPluginName[i]=((wchar_t *)pd->pFunction)[i];
    wszPluginName[i]='\0';
  }

  if (bOldWindows)
  {
    HMODULE hKernel32;

    hKernel32=GetModuleHandleA("kernel32.dll");
    GetCPInfoExAPtr=(BOOL (WINAPI *)(UINT, DWORD, LPCPINFOEXA))GetProcAddress(hKernel32, "GetCPInfoExA");
    ReadOptionsA();
  }
  else
  {
    HMODULE hKernel32;

    hKernel32=GetModuleHandleW(L"kernel32.dll");
    GetCPInfoExWPtr=(BOOL (WINAPI *)(UINT, DWORD, LPCPINFOEXW))GetProcAddress(hKernel32, "GetCPInfoExW");
    ReadOptionsW();
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
  }
  return TRUE;
}
